@@ -24,10 +24,16 @@ pub struct Context {
      * e.g. en, en_US
      */
     pub locale: Option<String>,
-    pub geolocation: Option<String>, // A two-character ISO 3166-1 country code
+    pub country_code: Option<String>, // A two-character ISO 3166-1 country code
     pub device_id: Option<String>,
     pub origin_service: Option<String>,
-    pub other_data: Option<HashMap<String, String>>,
+    pub user_is_employee: Option<bool>,
+    pub logged_in: Option<bool>,
+    pub app_name: Option<String>,
+    pub build_number: Option<i32>,
+    pub authentication_token: Option<String>,
+    pub request_url: Option<String>,
+    pub cookie_created_timestamp: Option<f64>,
 }
 
 impl Context {
@@ -138,7 +144,6 @@ pub struct EqStruct {
 impl EqStruct {
     fn eval(&self, ctx: &Context) -> bool {
         // FIXME: this makes values trump value, which is gross.
-        println!("ctx: {:#?}, tt: {:#?}", ctx, self);
         return match &self.values {
             Some(vl) => vl.iter().any(|x| ctx.cmp(&self.field, x)),
             None => match &self.value {
@@ -229,6 +234,52 @@ impl Decider {
         }
     }
 
+    pub fn get_int(&self, feature_name: String, ctx: &Context) -> Result<i64, DeciderFailType> {
+        let f = self.feature_by_name(feature_name)?;
+        let res = self.decide(&f, ctx);
+        match self.validate_dc_value(res) {
+            Ok(value) => match value {
+                serde_json::Value::Number(b) => match b.as_i64() {
+                    Some(int) => Ok(int),
+                    _ => Err(DeciderFailType::NumberDeserializationError),
+                },
+                _ => Err(DeciderFailType::DcTypeMismatch),
+            },
+            Err(e) => Err(e),
+        }
+    }
+
+    pub fn get_float(&self, feature_name: String, ctx: &Context) -> Result<f64, DeciderFailType> {
+        let f = self.feature_by_name(feature_name)?;
+        let res = self.decide(&f, ctx);
+        match self.validate_dc_value(res) {
+            Ok(value) => match value {
+                serde_json::Value::Number(b) => match b.as_f64() {
+                    Some(float) => Ok(float),
+                    _ => Err(DeciderFailType::NumberDeserializationError),
+                },
+                _ => Err(DeciderFailType::DcTypeMismatch),
+            },
+            Err(e) => Err(e),
+        }
+    }
+
+    pub fn get_string(
+        &self,
+        feature_name: String,
+        ctx: &Context,
+    ) -> Result<String, DeciderFailType> {
+        let f = self.feature_by_name(feature_name)?;
+        let res = self.decide(&f, ctx);
+        match self.validate_dc_value(res) {
+            Ok(value) => match value {
+                serde_json::Value::String(s) => Ok(s),
+                _ => Err(DeciderFailType::DcTypeMismatch),
+            },
+            Err(e) => Err(e),
+        }
+    }
+
     pub fn get_map(
         &self,
         feature_name: String,
@@ -275,20 +326,12 @@ impl Decider {
         // account for ill-defined Features or missing data on the Context. Probably
         // yes, but currently Decision also has an option on name/bucketing.  Maybe
         // decide should return a Result<Decision>?
-        println!(
-            "calling decide on feature name: {:#?} with context:{:#?}",
-            f.name, ctx
-        );
-
         for fun in &self.decisionmakers {
             let res = fun(f, ctx)?; // Should I let errors bubble up?  probably not.
             match res {
                 DecisionmakerResult::None => return Ok(None),
-                DecisionmakerResult::Pass(s) => {
-                    println!("decisionmaker passed: {}", s);
-                }
+                DecisionmakerResult::Pass(_s) => (),
                 DecisionmakerResult::Decided(d) => {
-                    println!("got a decision:{:#?}", d);
                     return Ok(Some(d));
                 }
             }
@@ -417,6 +460,7 @@ pub enum DeciderFailType {
     FeatureNotFound,
     InvalidFeature,
     DcTypeMismatch,
+    NumberDeserializationError,
     MapDeserializationError(String),
     InvalidDecisionMaker(String),
     IoError(std::io::Error),
@@ -431,6 +475,9 @@ impl fmt::Display for DeciderFailType {
             DeciderFailType::DcTypeMismatch => {
                 write!(f, "Dynamic Configuartion Feature type mismatch.")
             }
+            DeciderFailType::NumberDeserializationError => {
+                write!(f, "Number deserialization failed.")
+            }
             DeciderFailType::MapDeserializationError(err_str) => {
                 write!(f, "Map deserialization failed: {:#?}", err_str.clone())
             }
@@ -637,6 +684,14 @@ mod tests {
     use proptest::prelude::*;
     use serde_json::Error;
 
+    enum Dc {
+        Bool(Option<serde_json::Value>),
+        Int(Option<serde_json::Value>),
+        Float(Option<serde_json::Value>),
+        String(Option<serde_json::Value>),
+        Map(Option<String>),
+    }
+
     fn make_feature() -> Feature {
         let v = Variant {
             name: "enabled".to_string(),
@@ -663,38 +718,53 @@ mod tests {
         }
     }
 
-    fn make_bool_dynamic_config(bool: Option<serde_json::Value>) -> Feature {
-        let b = match bool {
-            None => serde_json::Value::Bool(true),
-            Some(b) => b,
+    fn make_dynamic_config(dc: Dc) -> Feature {
+        let name_str: String;
+
+        let value: serde_json::Value = match dc {
+            Dc::Bool(val) => {
+                name_str = "bool".to_string();
+                match val {
+                    None => serde_json::Value::Bool(true),
+                    Some(b) => b,
+                }
+            }
+            Dc::Int(val) => {
+                name_str = "int".to_string();
+                match val {
+                    None => serde_json::Value::from(1),
+                    Some(i) => i,
+                }
+            }
+            Dc::Float(val) => {
+                name_str = "float".to_string();
+                match val {
+                    None => serde_json::Value::from(2.0),
+                    Some(f) => f,
+                }
+            }
+            Dc::String(val) => {
+                name_str = "string".to_string();
+                match val {
+                    None => serde_json::Value::from("some_string"),
+                    Some(s) => s,
+                }
+            }
+            Dc::Map(val) => {
+                name_str = "map".to_string();
+                match &val {
+                    None => serde_json::from_str(r#"{"v":{"nested_map": {"w":false,"x": 1,"y":"some_string","z":3.0}},"w":false,"x": 1,"y":"some_string","z":3.0}"#).unwrap(),
+                    Some(s) => serde_json::from_str(s).unwrap()
+                }
+            }
         };
 
         Feature {
             id: 2,
-            name: "bool_dynamic_config".to_string(),
+            name: name_str + "_dynamic_config",
             enabled: true,
             version: 1,
-            value: Some(b),
-            targeting: None,
-            overrides: None,
-            platform_bitmask: 0,
-            variant_set: None,
-        }
-    }
-
-    fn make_map_dynamic_config(json: Option<String>) -> Feature {
-        let default_map_dc = r#"{"v":{"nested_map": {"w":false,"x": 1,"y":"some_string","z":3.0}},"w":false,"x": 1,"y":"some_string","z":3.0}"#;
-        let s = match &json {
-            None => default_map_dc,
-            Some(str) => str,
-        };
-
-        Feature {
-            id: 2,
-            name: "map_dynamic_config".to_string(),
-            enabled: true,
-            version: 1,
-            value: Some(serde_json::from_str(s).unwrap()),
+            value: Some(value),
             targeting: None,
             overrides: None,
             platform_bitmask: 0,
@@ -740,9 +810,15 @@ mod tests {
             user_id: "795244".to_string(),
             locale: Some("US".to_string()),
             device_id: None,
-            geolocation: None,
+            country_code: None,
             origin_service: None,
-            other_data: None,
+            user_is_employee: None,
+            logged_in: None,
+            app_name: None,
+            build_number: None,
+            authentication_token: None,
+            request_url: None,
+            cookie_created_timestamp: None,
         };
         return match json {
             None => Ok(c),
@@ -803,8 +879,11 @@ mod tests {
         };
         let dcv = match dcvo {
             None => vec![
-                make_bool_dynamic_config(None),
-                make_map_dynamic_config(None),
+                make_dynamic_config(Dc::Bool(None)),
+                make_dynamic_config(Dc::Int(None)),
+                make_dynamic_config(Dc::Float(None)),
+                make_dynamic_config(Dc::String(None)),
+                make_dynamic_config(Dc::Map(None)),
             ],
             Some(dc) => dc,
         };
@@ -920,7 +999,11 @@ mod tests {
     #[test]
     fn get_bool_works() -> Result<(), DeciderFailType> {
         let ctx = make_ctx(None)?;
-        let d = build_decider(Some("darkmode value".to_string()), None, None)?;
+        let d = build_decider(
+            Some("darkmode fractional_availability value".to_string()),
+            None,
+            None,
+        )?;
         let dc_res = d.get_bool("bool_dynamic_config".to_string(), &ctx);
         match dc_res {
             Ok(res) => {
@@ -939,7 +1022,11 @@ mod tests {
     #[test]
     fn get_bool_errors_on_missing_name() -> Result<(), DeciderFailType> {
         let ctx = make_ctx(None)?;
-        let d = build_decider(None, None, None)?;
+        let d = build_decider(
+            Some("darkmode fractional_availability value".to_string()),
+            None,
+            None,
+        )?;
         let r = d.get_bool("missing".to_string(), &ctx);
         match r {
             Ok(_) => panic!(),
@@ -948,14 +1035,133 @@ mod tests {
         Ok(())
     }
 
+    #[test]
+    fn get_int_works() -> Result<(), DeciderFailType> {
+        let ctx = make_ctx(None)?;
+        let d = build_decider(
+            Some("darkmode fractional_availability value".to_string()),
+            None,
+            None,
+        )?;
+        let dc_res = d.get_int("int_dynamic_config".to_string(), &ctx);
+        match dc_res {
+            Ok(res) => {
+                if res == 1 {
+                    println!("got expected int: {:#?}", res)
+                } else {
+                    panic!()
+                }
+            }
+            _ => panic!(),
+        }
+
+        Ok(())
+    }
+
+    #[test]
+    fn get_int_errors_on_missing_name() -> Result<(), DeciderFailType> {
+        let ctx = make_ctx(None)?;
+        let d = build_decider(
+            Some("darkmode fractional_availability value".to_string()),
+            None,
+            None,
+        )?;
+        let r = d.get_int("missing".to_string(), &ctx);
+        match r {
+            Ok(_) => panic!(),
+            Err(e) => println!("got expected error: {:#?}", e),
+        }
+        Ok(())
+    }
+
+    #[test]
+    fn get_float_works() -> Result<(), DeciderFailType> {
+        let ctx = make_ctx(None)?;
+        let d = build_decider(
+            Some("darkmode fractional_availability value".to_string()),
+            None,
+            None,
+        )?;
+        let dc_res = d.get_float("float_dynamic_config".to_string(), &ctx);
+        match dc_res {
+            Ok(res) => {
+                if res == 2.0 {
+                    println!("got expected float: {:#?}", res)
+                } else {
+                    panic!()
+                }
+            }
+            _ => panic!(),
+        }
+
+        Ok(())
+    }
+
+    #[test]
+    fn get_float_errors_on_missing_name() -> Result<(), DeciderFailType> {
+        let ctx = make_ctx(None)?;
+        let d = build_decider(
+            Some("darkmode fractional_availability value".to_string()),
+            None,
+            None,
+        )?;
+        let r = d.get_float("missing".to_string(), &ctx);
+        match r {
+            Ok(_) => panic!(),
+            Err(e) => println!("got expected error: {:#?}", e),
+        }
+        Ok(())
+    }
+
+    #[test]
+    fn get_string_works() -> Result<(), DeciderFailType> {
+        let ctx = make_ctx(None)?;
+        let d = build_decider(
+            Some("darkmode fractional_availability value".to_string()),
+            None,
+            None,
+        )?;
+        let dc_res = d.get_string("string_dynamic_config".to_string(), &ctx);
+        match dc_res {
+            Ok(res) => {
+                if res == "some_string" {
+                    println!("got expected string: {:#?}", res)
+                } else {
+                    panic!()
+                }
+            }
+            _ => panic!(),
+        }
+
+        Ok(())
+    }
+
+    #[test]
+    fn get_string_errors_on_missing_name() -> Result<(), DeciderFailType> {
+        let ctx = make_ctx(None)?;
+        let d = build_decider(
+            Some("darkmode fractional_availability value".to_string()),
+            None,
+            None,
+        )?;
+        let r = d.get_string("missing".to_string(), &ctx);
+        match r {
+            Ok(_) => panic!(),
+            Err(e) => println!("got expected error: {:#?}", e),
+        }
+        Ok(())
+    }
+
     #[test]
     fn get_map_works() -> Result<(), DeciderFailType> {
         let ctx = make_ctx(None)?;
         let map_dc_str = r#"{"v":{"nested_map": {"w":false,"x": 1,"y":"some_string","z":3.0}},"w":false,"x": 1,"y":"some_string","z":3.0}"#;
         let d = build_decider(
-            Some("darkmode value".to_string()),
+            Some("darkmode fractional_availability value".to_string()),
             None,
-            Some(vec![make_map_dynamic_config(Some(map_dc_str.to_string()))]),
+            Some(vec![make_dynamic_config(Dc::Map(Some(
+                map_dc_str.to_string(),
+            )))]),
         )?;
         let map = serde_json::from_str(map_dc_str).unwrap();
         let dc_res = d.get_map("map_dynamic_config".to_string(), &ctx);
@@ -976,7 +1182,11 @@ mod tests {
     #[test]
     fn get_map_errors_on_missing_name() -> Result<(), DeciderFailType> {
         let ctx = make_ctx(None)?;
-        let d = build_decider(None, None, None)?;
+        let d = build_decider(
+            Some("darkmode fractional_availability value".to_string()),
+            None,
+            None,
+        )?;
         let r = d.get_map("missing".to_string(), &ctx);
         match r {
             Ok(_) => panic!(),
